@@ -1,7 +1,7 @@
 // drumkv1.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,6 +28,7 @@
 
 // forward declarations
 class drumkv1_impl;
+class drumkv1_port;
 class drumkv1_elem;
 class drumkv1_element;
 class drumkv1_sample;
@@ -45,7 +46,7 @@ public:
 
 	drumkv1(uint16_t nchannels = 2, float srate = 44100.0f);
 
-	~drumkv1();
+	virtual ~drumkv1();
 
 	void setChannels(uint16_t nchannels);
 	uint16_t channels() const;
@@ -53,13 +54,19 @@ public:
 	void setSampleRate(float srate);
 	float sampleRate() const;
 
+	void setBufferSize(uint32_t nsize);
+	uint32_t bufferSize() const;
+
 	drumkv1_element *addElement(int key);
 	drumkv1_element *element(int key) const;
 	void removeElement(int key);
 
 	void setCurrentElement(int key);
+	void setCurrentElementEx(int key);
 	int currentElement() const;
 
+	void currentElementTest();
+
 	void clearElements();
 
 	void setSampleFile(const char *pszSampleFile);
@@ -70,6 +77,9 @@ public:
 	void setReverse(bool bReverse);
 	bool isReverse() const;
 
+	void setTempo(float bpm);
+	float tempo() const;
+
 	enum ParamIndex	 {
 
 		GEN1_SAMPLE = 0,
@@ -89,7 +99,9 @@ public:
 		DCF1_DECAY2,
 		LFO1_SHAPE,
 		LFO1_WIDTH,
+		LFO1_BPM,
 		LFO1_RATE,
+		LFO1_SYNC,
 		LFO1_SWEEP,
 		LFO1_PITCH,
 		LFO1_CUTOFF,
@@ -107,6 +119,7 @@ public:
 		DCA1_DECAY2,
 		OUT1_WIDTH,
 		OUT1_PANNING,
+		OUT1_FXSEND,
 		OUT1_VOLUME,
 
 		NUM_ELEMENT_PARAMS,
@@ -136,8 +149,6 @@ public:
 		DEL1_DELAY,
 		DEL1_FEEDB,
 		DEL1_BPM,
-		DEL1_BPMSYNC,
-		DEL1_BPMHOST,
 		REV1_WET,
 		REV1_ROOM,
 		REV1_DAMP,
@@ -150,7 +161,7 @@ public:
 	};
 
 	void setParamPort(ParamIndex index, float *pfParam);
-	float *paramPort(ParamIndex index) const;
+	drumkv1_port *paramPort(ParamIndex index) const;
 
 	void setParamValue(ParamIndex index, float fValue);
 	float paramValue(ParamIndex index) const;
@@ -164,6 +175,11 @@ public:
 	void process_midi(uint8_t *data, uint32_t size);
 	void process(float **ins, float **outs, uint32_t nframes);
 
+protected:
+
+	virtual void selectSample(int key) = 0;
+	virtual void updateSample() = 0;
+
 private:
 
 	drumkv1_impl *m_pImpl;
@@ -190,8 +206,8 @@ public:
 	void setReverse(bool bReverse);
 	bool isReverse() const;
 
-	void setParamPort(drumkv1::ParamIndex index, float *pfParam = 0);
-	float *paramPort(drumkv1::ParamIndex index);
+	void setParamPort(drumkv1::ParamIndex index, float *pfParam);
+	drumkv1_port *paramPort(drumkv1::ParamIndex index);
 
 	void setParamValue(drumkv1::ParamIndex index, float fValue, int pset = 1);
 	float paramValue(drumkv1::ParamIndex index, int pset = 1);
