@@ -1,7 +1,7 @@
 // drumkv1_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -104,7 +104,7 @@ private:
 
 
 //-------------------------------------------------------------------------
-// jack process callback.
+// JACK process callback.
 
 static
 int drumkv1_jack_process ( jack_nframes_t nframes, void *arg )
@@ -112,6 +112,16 @@ int drumkv1_jack_process ( jack_nframes_t nframes, void *arg )
 	return static_cast<drumkv1_jack *> (arg)->process(nframes);
 }
 
+//----------------------------------------------------------------------
+// JACK buffer-size change callback.
+
+static int drumkv1_jack_buffer_size ( jack_nframes_t nframes, void *arg )
+{
+	static_cast<drumkv1_jack *> (arg)->setBufferSize(nframes);
+
+	return 0;
+}
+
 
 #ifdef CONFIG_JACK_SESSION
 
@@ -164,8 +174,6 @@ drumkv1_jack::drumkv1_jack (void) : drumkv1(2)
 	m_alsa_thread  = NULL;
 #endif
 
-	m_bpm = 0.0f;
-
 	drumkv1::programs()->enabled(true);
 	drumkv1::controls()->enabled(true);
 
@@ -201,18 +209,12 @@ int drumkv1_jack::process ( jack_nframes_t nframes )
 			::jack_port_get_buffer(m_audio_outs[k], nframes));
 	}
 
-	const float bpm_sync = paramValue(drumkv1::DEL1_BPMSYNC);
-	if (bpm_sync > 0.0f) {
-		const float bpm_host = paramValue(drumkv1::DEL1_BPMHOST);
-		if (bpm_host > 0.0f) {
-			jack_position_t pos;
-			jack_transport_query(m_client, &pos);
-			if (pos.valid & JackPositionBBT) {
-				const float bpm = float(pos.beats_per_minute);
-				if (::fabs(bpm_host - bpm) > 0.01f)
-					setParamValue(drumkv1::DEL1_BPMHOST, bpm);
-			}
-		}
+	jack_position_t pos;
+	jack_transport_query(m_client, &pos);
+	if (pos.valid & JackPositionBBT) {
+		const float host_bpm = float(pos.beats_per_minute);
+		if (::fabsf(host_bpm - drumkv1::tempo()) > 0.001f)
+			drumkv1::setTempo(host_bpm);
 	}
 
 	uint32_t ndelta = 0;
@@ -224,12 +226,14 @@ int drumkv1_jack::process ( jack_nframes_t nframes )
 		for (uint32_t n = 0; n < nevents; ++n) {
 			jack_midi_event_t event;
 			::jack_midi_event_get(&event, midi_in, n);
-			uint32_t nread = event.time - ndelta;
-			if (nread > 0) {
-				drumkv1::process(ins, outs, nread);
-				for (uint16_t k = 0; k < nchannels; ++k) {
-					ins[k]  += nread;
-					outs[k] += nread;
+			if (event.time > ndelta) {
+				const uint32_t nread = event.time - ndelta;
+				if (nread > 0) {
+					drumkv1::process(ins, outs, nread);
+					for (uint16_t k = 0; k < nchannels; ++k) {
+						ins[k]  += nread;
+						outs[k] += nread;
+					}
 				}
 			}
 			ndelta = event.time;
@@ -268,7 +272,8 @@ int drumkv1_jack::process ( jack_nframes_t nframes )
 	}
 #endif // CONFIG_ALSA_MIDI
 
-	drumkv1::process(ins, outs, nframes - ndelta);
+	if (nframes > ndelta)
+		drumkv1::process(ins, outs, nframes - ndelta);
 
 	return 0;
 }
@@ -340,6 +345,12 @@ void drumkv1_jack::open ( const char *client_id )
 	}
 #endif	// CONFIG_ALSA_MIDI
 
+	// setup any local, initial buffers...
+	drumkv1::setBufferSize(::jack_get_buffer_size(m_client));
+
+	jack_set_buffer_size_callback(m_client,
+		drumkv1_jack_buffer_size, this);
+
 	// set process callbacks...
 	::jack_set_process_callback(m_client,
 		drumkv1_jack_process, this);
@@ -456,17 +467,6 @@ void drumkv1_jack::close (void)
 }
 
 
-void drumkv1_jack::setParamValue ( drumkv1::ParamIndex index, float fValue )
-{
-	m_params[index] = fValue;
-}
-
-float drumkv1_jack::paramValue ( drumkv1::ParamIndex index ) const
-{
-	return m_params[index];
-}
-
-
 #ifdef CONFIG_ALSA_MIDI
 
 // alsa sequencer client.
@@ -596,6 +596,21 @@ void drumkv1_jack::sessionEvent ( void *pvSessionArg )
 #endif	// CONFIG_JACK_SESSION
 
 
+void drumkv1_jack::selectSample ( int key )
+{
+	// something to do...
+	drumkv1::setCurrentElementEx(key);
+
+	drumkv1_sched::sync_notify(this, drumkv1_sched::Sample, 0);
+}
+
+
+void drumkv1_jack::updateSample (void)
+{
+	// nothing to do...
+}
+
+
 //-------------------------------------------------------------------------
 // drumkv1_jack_application -- Singleton application instance.
 //
@@ -696,11 +711,11 @@ bool drumkv1_jack_application::setup (void)
 
 	if (m_bGui) {
 		m_pWidget = new drumkv1widget_jack(m_pDrumk);
+		m_pWidget->show();
 		if (m_presets.isEmpty())
 			m_pWidget->initPreset();
 		else
 			m_pWidget->loadPreset(m_presets.first());
-		m_pWidget->show();
 	}
 	else
 	if (!m_presets.isEmpty())
