@@ -1,7 +1,7 @@
 // drumkv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -107,6 +107,8 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QStringList slopes;
 	slopes << tr("12dB/oct");
 	slopes << tr("24dB/oct");
+	slopes << tr("Biquad");
+	slopes << tr("Formant");
 
 	m_ui.Dcf1SlopeKnob->insertItems(0, slopes);
 
@@ -132,6 +134,8 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.Gen1ReverseKnob->insertItems(0, states);
 
+	m_ui.Lfo1SyncKnob->insertItems(0, states);
+
 	m_ui.Dyn1CompressKnob->insertItems(0, states);
 	m_ui.Dyn1LimiterKnob->insertItems(0, states);
 
@@ -145,6 +149,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	const QString& sAuto = tr("Auto");
 	m_ui.Gen1EnvTimeKnob->setSpecialValueText(sAuto);
+	m_ui.Lfo1BpmKnob->setSpecialValueText(sAuto);
 	m_ui.Del1BpmKnob->setSpecialValueText(sAuto);
 
 	// Wave integer widths.
@@ -171,6 +176,10 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Dcf1EnvelopeKnob->setMaximum(+1.0f);
 
 	// LFO parameter limits.
+	m_ui.Lfo1BpmKnob->setScale(1.0f);
+	m_ui.Lfo1BpmKnob->setMinimum(0.0f);
+	m_ui.Lfo1BpmKnob->setMaximum(360.0f);
+	m_ui.Lfo1BpmKnob->setSingleStep(1.0f);
 	m_ui.Lfo1SweepKnob->setMinimum(-1.0f);
 	m_ui.Lfo1SweepKnob->setMaximum(+1.0f);
 	m_ui.Lfo1CutoffKnob->setMinimum(-1.0f);
@@ -194,7 +203,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	// Effects (delay BPM)
 	m_ui.Del1BpmKnob->setScale(1.0f);
-	m_ui.Del1BpmKnob->setMinimum(3.6f);
+	m_ui.Del1BpmKnob->setMinimum(0.0f);
 	m_ui.Del1BpmKnob->setMaximum(360.0f);
 	m_ui.Del1BpmKnob->setSingleStep(1.0f);
 
@@ -268,7 +277,9 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	// LFO1
 	setParamKnob(drumkv1::LFO1_SHAPE,   m_ui.Lfo1ShapeKnob);
 	setParamKnob(drumkv1::LFO1_WIDTH,   m_ui.Lfo1WidthKnob);
+	setParamKnob(drumkv1::LFO1_BPM,     m_ui.Lfo1BpmKnob);
 	setParamKnob(drumkv1::LFO1_RATE,    m_ui.Lfo1RateKnob);
+	setParamKnob(drumkv1::LFO1_SYNC,    m_ui.Lfo1SyncKnob);
 	setParamKnob(drumkv1::LFO1_PANNING, m_ui.Lfo1PanningKnob);
 	setParamKnob(drumkv1::LFO1_VOLUME,  m_ui.Lfo1VolumeKnob);
 	setParamKnob(drumkv1::LFO1_CUTOFF,  m_ui.Lfo1CutoffKnob);
@@ -367,6 +378,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	// OUT1
 	setParamKnob(drumkv1::OUT1_WIDTH,   m_ui.Out1WidthKnob);
 	setParamKnob(drumkv1::OUT1_PANNING, m_ui.Out1PanningKnob);
+	setParamKnob(drumkv1::OUT1_FXSEND,  m_ui.Out1FxSendKnob);
 	setParamKnob(drumkv1::OUT1_VOLUME,  m_ui.Out1VolumeKnob);
 
 	// Reverb (stereo-)width limits.
@@ -396,10 +408,6 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(drumkv1::DEL1_FEEDB, m_ui.Del1FeedbKnob);
 	setParamKnob(drumkv1::DEL1_BPM,   m_ui.Del1BpmKnob);
 
-	QObject::connect(m_ui.Del1BpmKnob,
-		SIGNAL(valueChanged(float)),
-		SLOT(bpmSyncChanged()));
-
 	// Reverb
 	setParamKnob(drumkv1::REV1_WET,   m_ui.Rev1WetKnob);
 	setParamKnob(drumkv1::REV1_ROOM,  m_ui.Rev1RoomKnob);
@@ -623,9 +631,8 @@ void drumkv1widget::updateParamEx ( drumkv1::ParamIndex index, float fValue )
 		break;
 	}
 #endif
-	case drumkv1::DEL1_BPMSYNC:
-		if (fValue > 0.0f)
-			m_ui.Del1BpmKnob->setValue(0.0f);
+	case drumkv1::DCF1_SLOPE:
+		m_ui.Dcf1TypeKnob->setEnabled(int(fValue) != 3); // !Formant
 		// Fall thru...
 	default:
 		break;
@@ -668,13 +675,16 @@ void drumkv1widget::resetParams (void)
 
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i) {
 		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		if (index == drumkv1::GEN1_SAMPLE)
+			continue;
 		float fValue = drumkv1_param::paramDefaultValue(index);
 		drumkv1widget_knob *pKnob = paramKnob(index);
 		if (pKnob)
 			fValue = pKnob->defaultValue();
 		setParamValue(index, fValue);
 		updateParam(index, fValue);
-		m_params_ab[index] = fValue;
+	//	updateParamEx(index, fValue);
+		m_params_ab[i] = fValue;
 	}
 
 	m_ui.StatusBar->showMessage(tr("Reset preset"), 5000);
@@ -701,6 +711,8 @@ void drumkv1widget::swapParams ( bool bOn )
 		if (element) {
 			for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
 				const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+				if (index == drumkv1::GEN1_SAMPLE)
+					continue;
 				drumkv1widget_knob *pKnob = paramKnob(index);
 				if (pKnob) {
 					pKnob->setDefaultValue(element->paramValue(index, 0));
@@ -714,20 +726,24 @@ void drumkv1widget::swapParams ( bool bOn )
 		if (element) {
 			for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
 				const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-				m_params_ab[index] = element->paramValue(index);
+				if (index == drumkv1::GEN1_SAMPLE)
+					continue;
+				m_params_ab[i] = element->paramValue(index);
 			}
 		}
 	}
 
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i) {
 		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		if (index == drumkv1::GEN1_SAMPLE)
+			continue;
 		drumkv1widget_knob *pKnob = paramKnob(index);
 		if (pKnob) {
 			const float fOldValue = pKnob->value();
-			const float fNewValue = m_params_ab[index];
+			const float fNewValue = m_params_ab[i];
 			setParamValue(index, fNewValue);
 			updateParam(index, fNewValue);
-			m_params_ab[index] = fOldValue;
+			m_params_ab[i] = fOldValue;
 		}
 	}
 
@@ -755,13 +771,15 @@ void drumkv1widget::updateParamValues ( uint32_t nparams )
 
 	for (uint32_t i = 0; i < nparams; ++i) {
 		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		if (index == drumkv1::GEN1_SAMPLE)
+			continue;
 		const float fValue = (pDrumkUi
-			? pDrumkUi->paramValue(index)
-			: drumkv1_param::paramDefaultValue(index));
+		    ? pDrumkUi->paramValue(index)
+		    : drumkv1_param::paramDefaultValue(index));
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
 	//	updateParamEx(index, fValue);
-		m_params_ab[index] = fValue;
+		m_params_ab[i] = fValue;
 	}
 }
 
@@ -773,10 +791,13 @@ void drumkv1widget::resetParamValues ( uint32_t nparams )
 
 	for (uint32_t i = 0; i < nparams; ++i) {
 		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		if (index == drumkv1::GEN1_SAMPLE)
+			continue;
 		const float fValue = drumkv1_param::paramDefaultValue(index);
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
-		m_params_ab[index] = fValue;
+	//	updateParamEx(index, fValue);
+		m_params_ab[i] = fValue;
 	}
 }
 
@@ -785,7 +806,10 @@ void drumkv1widget::resetParamValues ( uint32_t nparams )
 void drumkv1widget::resetParamKnobs ( uint32_t nparams )
 {
 	for (uint32_t i = 0; i < nparams; ++i) {
-		drumkv1widget_knob *pKnob = paramKnob(drumkv1::ParamIndex(i));
+		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		if (index == drumkv1::GEN1_SAMPLE)
+			continue;
+		drumkv1widget_knob *pKnob = paramKnob(index);
 		if (pKnob)
 			pKnob->resetDefaultValue();
 	}
@@ -814,7 +838,6 @@ void drumkv1widget::newPreset (void)
 #endif
 
 	clearElements();
-
 	clearSampleFile();
 
 	resetParamKnobs(drumkv1::NUM_PARAMS);
@@ -840,7 +863,6 @@ void drumkv1widget::loadPreset ( const QString& sFilename )
 #endif
 
 	clearElements();
-
 	clearSampleFile();
 
 	resetParamKnobs(drumkv1::NUM_PARAMS);
@@ -925,24 +947,13 @@ void drumkv1widget::loadSampleFile ( const QString& sFilename )
 	if (pDrumkUi == NULL)
 		return;
 
-	const int note = currentNote();
-	if (note < 0)
+	const int iCurrentNote = currentNote();
+	if (iCurrentNote < 0)
 		return;
 
-	drumkv1_element *element = pDrumkUi->element(note);
-	if (element == NULL) {
-		element = pDrumkUi->addElement(note);
-		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-			const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-			const float fValue = drumkv1_param::paramDefaultValue(index);
-			element->setParamValue(index, fValue);
-		}
-		pDrumkUi->setCurrentElement(note);
-		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-			const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-			setParamValue(index, element->paramValue(index));
-		}
-		activateParamKnobs(true);
+	if (pDrumkUi->element(iCurrentNote) == NULL) {
+		pDrumkUi->addElement(iCurrentNote);
+		pDrumkUi->setCurrentElement(iCurrentNote);
 	}
 
 	pDrumkUi->setSampleFile(sFilename.toUtf8().constData());
@@ -1088,18 +1099,22 @@ void drumkv1widget::refreshElements (void)
 {
 	const bool bBlockSignals = m_ui.Elements->blockSignals(true);
 
+	drumkv1_ui *pDrumkUi = ui_instance();
 	if (m_ui.Elements->instance() == NULL)
-		m_ui.Elements->setInstance(ui_instance());
+		m_ui.Elements->setInstance(pDrumkUi);
 
 	int iCurrentNote = currentNote();
+	if (iCurrentNote < 0 && pDrumkUi)
+		iCurrentNote = pDrumkUi->currentElement();
+	if (iCurrentNote < 0)
+		iCurrentNote = 36; // Bass Drum 1 (default)
 
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_DEBUG_0
 	qDebug("drumkv1widget::refreshElements(%d)", iCurrentNote);
 #endif
 
 	m_ui.Elements->refresh();
 
-	if (iCurrentNote < 0) iCurrentNote = 36; // Bass Drum 1 (default)
 	m_ui.Elements->setCurrentIndex(iCurrentNote);
 	m_ui.Gen1Sample->setSampleName(completeNoteName(iCurrentNote));
 
@@ -1136,6 +1151,8 @@ void drumkv1widget::activateElement ( bool bOpenSample )
 		element = pDrumkUi->addElement(iCurrentNote);
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
 			const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+			if (index == drumkv1::GEN1_SAMPLE)
+				continue;
 			const float fValue = drumkv1_param::paramDefaultValue(index);
 			element->setParamValue(index, fValue, 0);
 			element->setParamValue(index, fValue);
@@ -1144,32 +1161,8 @@ void drumkv1widget::activateElement ( bool bOpenSample )
 
 	pDrumkUi->setCurrentElement(iCurrentNote);
 
-	resetParamKnobs(drumkv1::NUM_ELEMENT_PARAMS);
-
-	if (element) {
-		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
-			const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-			drumkv1widget_knob *pKnob = paramKnob(index);
-			if (pKnob)
-				pKnob->setDefaultValue(element->paramValue(index, 0));
-			const float fValue = element->paramValue(index);
-			setParamValue(index, fValue);
-			updateParam(index, fValue);
-		}
-		updateSample(pDrumkUi->sample());
-		refreshElements();
-	} else {
-		updateSample(NULL);
-		resetParamValues(drumkv1::NUM_ELEMENT_PARAMS);
-	}
-
-	activateParamKnobs(element != NULL);
-
-	const QString& sElementName = completeNoteName(iCurrentNote);
-	m_ui.StatusBar->showMessage(tr("Element: %1").arg(sElementName), 5000);
-
 	if (bOpenSample)
-		m_ui.Gen1Sample->openSample(sElementName);
+		m_ui.Gen1Sample->openSample(completeNoteName(iCurrentNote));
 }
 
 
@@ -1180,6 +1173,52 @@ void drumkv1widget::doubleClickElement (void)
 }
 
 
+// Update element (as current).
+void drumkv1widget::updateElement (void)
+{
+	resetParamKnobs(drumkv1::NUM_ELEMENT_PARAMS);
+
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi == NULL)
+		return;
+
+	const int iCurrentNote = pDrumkUi->currentElement();
+	const bool bBlockSignals = m_ui.Elements->blockSignals(true);
+	m_ui.Elements->setCurrentIndex(iCurrentNote);
+	m_ui.Elements->blockSignals(bBlockSignals);
+
+#ifdef CONFIG_DEBUG_0
+	qDebug("drumkv1widget::updateElement(%d)", iCurrentNote);
+#endif
+
+	++m_iUpdate;
+
+	drumkv1_element *element = pDrumkUi->element(iCurrentNote);
+	if (element) {
+		activateParamKnobs(true);
+		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
+			const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+			const float fValue = element->paramValue(index);
+			drumkv1widget_knob *pKnob = paramKnob(index);
+			if (pKnob) {
+				pKnob->setDefaultValue(element->paramValue(index, 0));
+				pKnob->setValue(fValue);
+			}
+			updateParam(index, fValue);
+			m_params_ab[i] = fValue;
+		}
+		updateSample(pDrumkUi->sample());
+		refreshElements();
+	} else {
+		updateSample(NULL);
+		resetParamValues(drumkv1::NUM_ELEMENT_PARAMS);
+		activateParamKnobs(false);
+	}
+
+	--m_iUpdate;
+}
+
+
 // Element sample loader.
 void drumkv1widget::loadSampleElement ( const QString& sFilename )
 {
@@ -1228,24 +1267,6 @@ void drumkv1widget::activateParamKnobsGroupBox (
 }
 
 
-// Delay BPM change.
-void drumkv1widget::bpmSyncChanged (void)
-{
-	if (m_iUpdate > 0)
-		return;
-
-	++m_iUpdate;
-	drumkv1_ui *pDrumkUi = ui_instance();
-	if (pDrumkUi) {
-		const bool bBpmSync0 = (pDrumkUi->paramValue(drumkv1::DEL1_BPMSYNC) > 0.0f);
-		const bool bBpmSync1 = m_ui.Del1BpmKnob->isSpecialValue();
-		if ((bBpmSync1 && !bBpmSync0) || (!bBpmSync1 && bBpmSync0))
-			pDrumkUi->setParamValue(drumkv1::DEL1_BPMSYNC, (bBpmSync1 ? 1.0f : 0.0f));
-	}
-	--m_iUpdate;
-}
-
-
 // Common context menu.
 void drumkv1widget::contextMenuRequest ( const QPoint& pos )
 {
@@ -1284,6 +1305,8 @@ void drumkv1widget::contextMenuRequest ( const QPoint& pos )
 // Preset status updater.
 void drumkv1widget::updateLoadPreset ( const QString& sPreset )
 {
+	resetSwapParams();
+
 //	refreshElements();
 	activateElement();
 
@@ -1334,7 +1357,7 @@ void drumkv1widget::updateSchedNotify ( int stype, int sid )
 			updateParamValues(drumkv1::NUM_PARAMS);
 			updateDirtyPreset(false);
 		} else {
-			updateSample(pDrumkUi->sample());
+			updateElement();
 		}
 		// Fall thru...
 	default:
